@@ -5,6 +5,7 @@ information.
 """
 
 import geopandas
+import pandas
 import pathlib
 import shapely
 import numpy
@@ -492,6 +493,311 @@ class Channel:
         return numpy.array([x_sampled, y_sampled])
 
 
+class InterpolateMeasuredElevations:
+    """Interpolate measured river bed elevations. Requires LineStrings of
+    riverbed elevations, and the LineStrings defining the left and right
+    riverbank."""
+
+    def __init__(
+        self,
+        riverbank_file: pathlib.Path,
+        measured_sections_file: pathlib.Path,
+        cross_section_spacing: int,
+        crs: int,
+    ):
+        """Setup for interpolating the measured riverbed elevations along
+        equally spaced cross sections."""
+        self.riverbanks = geopandas.read_file(riverbank_file)
+        self.measured_sections = geopandas.read_file(measured_sections_file)
+        self.cross_section_spacing = cross_section_spacing
+        self.crs = crs
+        self.check_valid_inputs()
+
+    def check_valid_inputs(self):
+        """Check expected entries in the input data"""
+
+        # Check expected riverbank information
+        if self.riverbanks.crs is None:
+            raise Exception("CRS required for self.measured_sections. Is None.")
+        if (
+            len(self.riverbanks.columns) != 2
+            and "Name" not in self.riverbanks.columns
+            and "geometry" not in self.riverbanks.columns
+        ):
+            raise Exception(
+                "Unexpected columns for self.riverbanks. Expecting"
+                " ['Name', 'geometry'], but got"
+                f" {self.riverbanks.columns}"
+            )
+
+        if (
+            len(self.riverbanks) != 2
+            and not (self.riverbanks["Name"] == "Right").any()
+            and not (self.riverbanks["Name"] == "Left").any()
+        ):
+            raise Exception(
+                "Unexpected row for self.riverbanks. Expecting two"
+                " rows with names ['Right', 'Left'], but got"
+                f" {self.riverbanks['Name']}"
+            )
+
+        # Check the riverbank lines are both in the same direction
+        if self.riverbanks.iloc[0].geometry.interpolate(0, normalized=True).distance(
+            self.riverbanks.loc[1].geometry.interpolate(0, normalized=True)
+        ) > self.riverbanks.iloc[0].geometry.interpolate(0, normalized=True).distance(
+            self.riverbanks.loc[1].geometry.interpolate(1, normalized=True)
+        ) or (
+            self.riverbanks.iloc[0]
+            .geometry.interpolate(1, normalized=True)
+            .distance(self.riverbanks.loc[1].geometry.interpolate(1, normalized=True))
+            > self.riverbanks.iloc[0]
+            .geometry.interpolate(1, normalized=True)
+            .distance(self.riverbanks.loc[1].geometry.interpolate(0, normalized=True))
+        ):
+            logging.Warning(
+                "It appears that the `right` and `left` banks are"
+                "defined in different directions based on the "
+                "relative distance between the endpoints. Please "
+                "define both as either upstream or downstream."
+            )
+
+        if self.riverbanks.crs is None:
+            raise Exception(
+                "self.riverbanks must have a valid crs. Instead"
+                f"has {self.riverbanks.crs}"
+            )
+
+        # Check valid measured cross section information
+        if self.measured_sections.crs is None:
+            raise Exception("CRS required for self.measured_sections. Is None.")
+        if not (self.measured_sections.geometry.type == "LineString").all():
+            raise Exception(
+                "The individual sections of the self.measured_sections must "
+                "all be of geometry type `LineString`. These are of type"
+                f"{self.measured_sections.geometry.type}"
+            )
+        # Set the CRS to be that defined for the output
+        self.riverbanks.to_crs(self.crs, inplace=True)
+        self.measured_sections.to_crs(self.crs, inplace=True)
+
+    def interpolate(self, samples_per_section: int, thalweg_centre: bool = True):
+        """Interpolate with equally spaced points along each cross section
+
+        Parameters
+        ----------
+
+        number_samples
+            The number of samples along each cross section."""
+
+        # Define river polygon and clip the measured sections
+        polygon = geopandas.GeoDataFrame(
+            geometry=[
+                shapely.geometry.Polygon(
+                    numpy.append(
+                        self.riverbanks[self.riverbanks["Name"] == "Right"]
+                        .iloc[0]
+                        .geometry.coords,
+                        self.riverbanks[self.riverbanks["Name"] == "Left"]
+                        .iloc[0]
+                        .geometry.coords[::-1],
+                        axis=0,
+                    )
+                )
+            ],
+            crs=self.riverbanks.crs,
+        )
+        # Clip measured sections
+        self.measured_sections = self.measured_sections.clip(polygon).sort_index()
+        if not (self.measured_sections.geometry.type == "LineString").all():
+            raise Exception(
+                "The individual sections of the self.measured_sections must "
+                "all be of geometry type `LineString` after clipping. Instead"
+                f"{self.measured_sections.geometry.type}"
+            )
+        measured_sections_exploded = self.measured_sections.apply(
+            lambda row: shapely.geometry.MultiPoint(row["geometry"].coords), axis=1
+        ).explode(index_parts=True)
+
+        # work out number of cross sections
+        n_cross_sections = round(
+            self.riverbanks.length.max() / self.cross_section_spacing
+        )
+
+        # Split riverbank polylines
+        normalised_sample_locations = (
+            numpy.arange(n_cross_sections + 1) * 1 / n_cross_sections
+        )
+        self.riverbanks["geometry"] = self.riverbanks["geometry"].apply(
+            lambda row: shapely.geometry.MultiPoint(
+                row.interpolate(normalised_sample_locations, normalized=True)
+            )
+        )
+
+        # Reshape to separate right and left column - split lines to points
+        cross_sections = self.riverbanks.explode(index_parts=True)
+        cross_sections = pandas.concat(
+            [
+                cross_sections[cross_sections["Name"] == "Right"]
+                .droplevel(0)
+                .geometry.rename("Right"),
+                cross_sections[cross_sections["Name"] == "Left"]
+                .droplevel(0)
+                .geometry.rename("Left"),
+            ],
+            axis=1,
+        )
+
+        # Create cross sections
+        cross_sections["Sections"] = cross_sections.apply(
+            lambda row: shapely.geometry.LineString([row["Right"], row["Left"]]), axis=1
+        )
+
+        # Define the node spacings along the cross sections
+        self.uniform_spacing(
+            samples_per_section=samples_per_section, cross_sections=cross_sections
+        )
+        # Note that in future could look at support for Thalweg centred spacing
+        # using the thalweg_centred_spacing function
+
+        # Explode the cross section points
+        cross_sections_exploded = (
+            cross_sections[["Sections"]]
+            .set_geometry("Sections", crs=self.riverbanks.crs)
+            .explode(index_parts=True)
+        )
+
+        # Map measured section values then interpolated down/up river
+        nearest_cross_section_point_indices = measured_sections_exploded.apply(
+            lambda row: row.distance(
+                cross_sections["Sections"].explode(index_parts=True)
+            ).idxmin()
+        ).unique()
+        cross_sections_exploded["z"] = numpy.nan
+        cross_sections_exploded.loc[
+            nearest_cross_section_point_indices, "z"
+        ] = cross_sections_exploded.loc[
+            nearest_cross_section_point_indices, "Sections"
+        ].apply(
+            lambda point: measured_sections_exploded.loc[
+                point.distance(measured_sections_exploded).idxmin()
+            ].z
+        )
+        cross_sections_exploded["z"] = (
+            cross_sections_exploded["z"]
+            .groupby(level=1, group_keys=False)
+            .apply(lambda group: group.interpolate(limit_direction="both"))
+        )
+
+        # Return river polygon and sampled cross sections
+        return polygon, cross_sections_exploded
+
+    def uniform_spacing(self, samples_per_section: int, cross_sections):
+        """Interpolate with equally spaced points along each cross section
+
+        Parameters
+        ----------
+
+        number_samples
+            The number of samples along each cross section."""
+
+        normalised_node_locations = numpy.arange(
+            0, 1 + 1.0 / samples_per_section, 1.0 / samples_per_section
+        )
+        cross_sections["Sections"] = cross_sections["Sections"].apply(
+            lambda line: shapely.geometry.MultiPoint(
+                line.interpolate(normalised_node_locations, normalized=True)
+            )
+        )
+
+    def thalweg_centred_spacing(
+        self,
+        samples_per_section: int,
+        cross_sections: geopandas.GeoDataFrame,
+        measured_sections_exploded: geopandas.GeoDataFrame,
+    ):
+        """Interpolate with equal number of equally spaced points on each side
+        of the Thalweg.
+
+        Parameters
+        ----------
+
+        number_samples
+            The number of samples along each cross section."""
+
+        # Calculate the normalised thalweg location at each measured section
+        def calculate_normalised_thalweg_location(indices: tuple, sections_exploded):
+            """Tuple of the section index, and the point index along the section"""
+            thalweg_point = sections_exploded[indices]
+            left_bank_distance = thalweg_point.distance(
+                sections_exploded.loc[indices[0]][0]
+            )
+            right_bank_distance = thalweg_point.distance(
+                sections_exploded.loc[indices[0]][
+                    len(sections_exploded.loc[indices[0]]) - 1
+                ]
+            )
+            normalised_thalweg_location = left_bank_distance / (
+                left_bank_distance + right_bank_distance
+            )
+
+            return normalised_thalweg_location
+
+        thalweg_indices = (
+            measured_sections_exploded.apply(lambda row: row.z)
+            .groupby(level=0)
+            .idxmin()
+        )
+        thalweg_points = measured_sections_exploded[thalweg_indices]
+
+        # Linearly interpolate the normalised Thalewg location along the cross sections
+        cross_sections["Thalweg ratio"] = numpy.nan
+        cross_sections.loc[
+            thalweg_indices.apply(
+                lambda row: cross_sections["Sections"]
+                .distance(thalweg_points[row])
+                .idxmin()
+            ),
+            "Thalweg ratio",
+        ] = thalweg_indices.apply(
+            lambda row: calculate_normalised_thalweg_location(
+                row, measured_sections_exploded
+            )
+        ).array
+        cross_sections["Thalweg ratio"].interpolate(
+            limit_direction="both", inplace=True
+        )
+
+        # Define equal sample locations on each side of the Thalweg
+        def split_line_to_nodes(line, thalweg_ratio, samples_per_section):
+            """Split line into n_nodes points half equally spaced on each side of the Thalweg"""
+            normalised_node_locations = numpy.concatenate(
+                [
+                    numpy.arange(samples_per_section)
+                    * thalweg_ratio
+                    / samples_per_section,
+                    numpy.arange(samples_per_section + 1)
+                    * (1 - thalweg_ratio)
+                    / samples_per_section
+                    + thalweg_ratio,
+                ]
+            )
+            return shapely.geometry.MultiPoint(
+                line.interpolate(normalised_node_locations, normalized=True)
+            )
+
+        # Apply the function for sampling along each cross section
+        cross_sections["Sections"] = cross_sections[
+            ["Sections", "Thalweg ratio"]
+        ].apply(
+            lambda row: split_line_to_nodes(
+                row["Sections"],
+                row["Thalweg ratio"],
+                samples_per_section=samples_per_section,
+            ),
+            axis=1,
+        )
+
+
 class ChannelCharacteristics:
     """A class to estimate the width, slope and other characteristics of a
     channel from a detailed DEM and a river network."""
@@ -687,38 +993,14 @@ class ChannelCharacteristics:
             cross_sections["min_z_centre"]
         )
 
-        # Set the water z value to use for width thresholding
-        cross_sections["min_z_water"] = cross_sections["min_z_centre_unimodal"]
-
         # Slope from the water surface - interpolate to fill any Nan
         cross_sections["slope"] = (
-            cross_sections["min_z_water"].diff() / self.cross_section_spacing
+            cross_sections["min_z_centre_unimodal"].diff() / self.cross_section_spacing
         )
         cross_sections["slope"] = cross_sections["slope"].interpolate(
             "index", limit_direction="both"
         )
 
-        # Slopes for a range of smoothings
-        for smoothing_distance in [50, 250, 500, 1000]:
-            # ensure odd number of samples so array length preserved
-            smoothing_samples = int(
-                numpy.ceil(smoothing_distance / self.cross_section_spacing)
-            )
-            smoothing_samples = int(smoothing_samples / 2) * 2 + 1
-            label = f"{smoothing_distance/1000}km"
-
-            # Smoothed min_z_centre_unimodal
-            cross_sections[
-                f"min_z_centre_unimodal_mean_{label}"
-            ] = self._rolling_mean_with_padding(
-                cross_sections["min_z_centre_unimodal"], smoothing_samples
-            )
-
-            # Smoothed slope
-            cross_sections[f"slope_mean_{label}"] = self._rolling_mean_with_padding(
-                cross_sections["slope"], smoothing_samples
-            )
-
     def _smooth_widths_and_thresholds(self, cross_sections: geopandas.GeoDataFrame):
         """Record the valid and reolling mean of the calculated thresholds and widths
 
@@ -735,75 +1017,14 @@ class ChannelCharacteristics:
         # Tidy up widths - pull out the valid widths
         cross_sections["valid_widths"] = cross_sections["widths"]
         cross_sections.loc[invalid_mask, "valid_widths"] = numpy.nan
-        widths_no_nan = cross_sections["valid_widths"].interpolate(
-            "index", limit_direction="both"
-        )
 
         # Flat widths
         cross_sections["valid_flat_widths"] = cross_sections["flat_widths"]
         cross_sections.loc[invalid_mask, "valid_flat_widths"] = numpy.nan
-        flat_widths_no_nan = cross_sections["valid_flat_widths"].interpolate(
-            "index", limit_direction="both"
-        )
 
         # Tidy up thresholds - pull out the valid thresholds
         cross_sections["valid_threhold"] = cross_sections["threshold"]
         cross_sections.loc[invalid_mask, "valid_threhold"] = numpy.nan
-        thresholds_no_nan = cross_sections["valid_threhold"].interpolate(
-            "index", limit_direction="both"
-        )
-
-        # Cycle through and caluclate the rolling mean
-        for smoothing_distance in [50, 150, 200, 250]:
-            # ensure odd number of samples so array length preserved
-            smoothing_samples = int(
-                numpy.ceil(smoothing_distance / self.cross_section_spacing)
-            )
-            smoothing_samples = int(smoothing_samples / 2) * 2 + 1
-            label = f"{smoothing_distance/1000}km"
-
-            # Apply the rolling mean to each
-            cross_sections[f"widths_mean_{label}"] = self._rolling_mean_with_padding(
-                widths_no_nan, smoothing_samples
-            )
-            cross_sections[
-                f"flat_widths_mean_{label}"
-            ] = self._rolling_mean_with_padding(flat_widths_no_nan, smoothing_samples)
-            cross_sections[
-                f"thresholds_mean_{label}"
-            ] = self._rolling_mean_with_padding(thresholds_no_nan, smoothing_samples)
-
-            """cross_sections[f'widths_Savgol_{label}'] = scipy.signal.savgol_filter(
-                cross_sections['widths'].interpolate('index', limit_direction='both'),
-                smoothing_samples,  # Ensure odd. number of samples included
-                3)  # Polynomial order"""
-
-    def _rolling_mean_with_padding(
-        self, data: geopandas.GeoSeries, number_of_samples: int
-    ) -> numpy.ndarray:
-        """Calculate the rolling mean of an array after padding the array with
-        the edge value to ensure the derivative is smooth.
-
-        Parameters
-        ----------
-
-        data
-            The array to pad then smooth.
-        number_of_samples
-            The width in samples of the averaging filter
-        """
-        assert (
-            number_of_samples > 0 and type(number_of_samples) == int
-        ), "Must be more than 0 and an int"
-        rolling_mean = (
-            numpy.convolve(
-                numpy.pad(data, int(number_of_samples / 2), "symmetric"),
-                numpy.ones(number_of_samples),
-                "valid",
-            )
-            / number_of_samples
-        )
-        return rolling_mean
 
     def sample_cross_sections(
         self, cross_sections: geopandas.GeoDataFrame, min_z_search_radius: float
@@ -929,7 +1150,7 @@ class ChannelCharacteristics:
             gnd_samples = cross_section_elevations["gnd_elevations"][j]
             veg_samples = cross_section_elevations["veg_elevations"][j]
             start_index = self.centre_index
-            z_water = cross_sections.iloc[j]["min_z_water"]
+            z_water = cross_sections.iloc[j]["min_z_centre_unimodal"]
 
             start_i, stop_i, channel_count = self.fixed_threshold_width(
                 gnd_samples=gnd_samples,
@@ -1015,7 +1236,7 @@ class ChannelCharacteristics:
             gnd_samples = cross_section_elevations["gnd_elevations"][j]
             veg_samples = cross_section_elevations["veg_elevations"][j]
             start_index = self.centre_index
-            z_water = cross_sections.iloc[j]["min_z_water"]
+            z_water = cross_sections.iloc[j]["min_z_centre_unimodal"]
 
             # Get width based on fixed threshold
             start_i, stop_i, channel_count = self.fixed_threshold_width(
@@ -1032,101 +1253,116 @@ class ChannelCharacteristics:
             widths["last_flat_bank_i"].append(stop_i)
             widths["flat_widths"].append((stop_i - start_i) * resolution)
 
-            # Iterate out from the fixed threshold width until the banks go down, or the
-            # max threshold is reached
-            maximum_z = z_water + maximum_threshold
-            if numpy.isnan(start_i) or numpy.isnan(stop_i):
-                # No valid width to begin with
+            # Iterate out from the fixed threshold width until the banks go
+            # down, or the max threshold is reached
+            if (
+                numpy.isnan(start_i)
+                or numpy.isnan(stop_i)
+                or numpy.isnan([gnd_samples[start_i], veg_samples[start_i]]).all()
+                or numpy.isnan([gnd_samples[stop_i], veg_samples[stop_i]]).all()
+            ):
+                # No valid width or elevations to begin with
                 dz_bankfull = numpy.nan
             else:
-                # Iterate out from the fixed threshold width until the banks go down,
-                # or the max threshold is reached
-                z_bankfull = numpy.nanmin(gnd_samples[[start_i, stop_i]])
+                # Calculate the threshold elevation (z)
+                maximum_z = z_water + maximum_threshold
+                # The bankfull indices - updated if moving out each iteration
                 start_i_bf = start_i
                 stop_i_bf = stop_i
                 dwidth = 1  # Change in width this iteration
+                # Calculate the initial end elevations - checked above not NaN
+                prev_start_elevation = numpy.nanmin(
+                    [gnd_samples[start_i], veg_samples[start_i]]
+                )
+                prev_stop_elevation = numpy.nanmin(
+                    [gnd_samples[stop_i], veg_samples[stop_i]]
+                )
 
-                while (
+                while (  # Stop when no move out or at end of section
                     start_i_bf > 0
                     and stop_i_bf < self.number_of_samples - 1
                     and dwidth > 0
                 ):
                     dwidth = 0
-
-                    # break if going down - TODO do after other side finished increasing
-                    if gnd_samples[start_i_bf - 1] < numpy.nanmax(
-                        [gnd_samples[start_i_bf], z_bankfull]
-                    ) or gnd_samples[stop_i_bf + 1] < numpy.nanmax(
-                        [gnd_samples[stop_i_bf], z_bankfull]
-                    ):
-                        break
-                    # if not, extend whichever bank is lower
-                    if gnd_samples[start_i_bf - 1] > gnd_samples[stop_i_bf + 1]:
-                        stop_i_bf += 1
-                        dwidth += 1
-                    elif gnd_samples[start_i_bf - 1] < gnd_samples[stop_i_bf + 1]:
-                        start_i_bf -= 1
-                        dwidth += 1
-                    elif gnd_samples[start_i_bf - 1] == gnd_samples[stop_i_bf + 1]:
-                        start_i_bf -= 1
-                        stop_i_bf += 1
-                        dwidth += 2
+                    # Get the current start elevation
+                    elevations = [
+                        gnd_samples[start_i_bf - 1],
+                        veg_samples[start_i_bf - 1],
+                    ]
+                    if numpy.isnan(elevations).all():
+                        start_elevation = numpy.nan
                     else:
-                        # extend if value is nan and not vegetated
-                        if numpy.isnan(gnd_samples[start_i_bf - 1]) and numpy.isnan(
-                            veg_samples[start_i_bf - 1]
-                        ):
-                            start_i_bf -= 1
-                            dwidth += 1
-                        if numpy.isnan(gnd_samples[stop_i_bf + 1]) and numpy.isnan(
-                            veg_samples[start_i_bf + 1]
-                        ):
-                            stop_i_bf += 1
-                            dwidth += 1
-                    # Break if the threshold has been meet before updating maz_z
+                        start_elevation = numpy.nanmin(elevations)
+                    # Get the current end elevation
+                    elevations = [
+                        gnd_samples[stop_i_bf + 1],
+                        veg_samples[stop_i_bf + 1],
+                    ]
+                    if numpy.isnan(elevations).all():
+                        stop_elevation = numpy.nan
+                    else:
+                        stop_elevation = numpy.nanmin(elevations)
+
+                    # Move outwards if all Nan, or lowest is going up
+                    if numpy.isnan(start_elevation):
+                        # start is all NaN - move out
+                        start_i_bf -= 1
+                        dwidth += 1
+                    elif numpy.isnan(stop_elevation):
+                        # stop is all NaN - move out
+                        stop_i_bf += 1
+                        dwidth += 1
+                    elif (
+                        start_elevation <= stop_elevation
+                        and start_elevation > prev_start_elevation
+                        and start_elevation > z_water + threshold
+                    ):
+                        # Start is lowest and going up and above min threshold
+                        start_i_bf -= 1
+                        dwidth += 1
+                        # update prev elevation
+                        prev_start_elevation = start_elevation
+                    elif (
+                        stop_elevation <= start_elevation
+                        and stop_elevation > prev_stop_elevation
+                        and stop_elevation > z_water + threshold
+                    ):
+                        # stop is lowest and going up  and above min threshold
+                        stop_i_bf += 1
+                        dwidth += 1
+                        # update prev elevation
+                        prev_stop_elevation = stop_elevation
+                    # Break if threshold is reached
                     if (
-                        gnd_samples[start_i_bf] >= maximum_z
-                        or gnd_samples[stop_i_bf] >= maximum_z
-                    ):
-                        break  # todo - set to the max specified threshold
-                    # Break if ground is nan, but there are vegatation returns
-                    if numpy.isnan(gnd_samples[start_i_bf]) and not numpy.isnan(
-                        veg_samples[start_i_bf]
-                    ):
-                        break  # todo - look at waikanae and consider checking the vegetation hights and only saying bank fill if over threshold or going down
-                    if numpy.isnan(gnd_samples[stop_i_bf]) and not numpy.isnan(
-                        veg_samples[stop_i_bf]
+                        not numpy.isnan([start_elevation, stop_elevation]).all()
+                        and numpy.nanmin([start_elevation, stop_elevation]).all()
+                        > maximum_z
                     ):
                         break
-                    # update maximum value so far
-                    if not numpy.isnan(
-                        [gnd_samples[start_i_bf], gnd_samples[stop_i_bf]]
-                    ).all():
-                        z_bankfull = max(
-                            z_bankfull,
-                            numpy.nanmin(
-                                [gnd_samples[start_i_bf], gnd_samples[stop_i_bf]]
-                            ),
-                        )
-                # Set the detected bankful values
-                dz_bankfull = z_bankfull - z_water
+
+                # Update with the final bankfull values
                 start_i = start_i_bf
                 stop_i = stop_i_bf
-            # assign the longest width
+                z_bankfull = min(
+                    max(prev_start_elevation, prev_stop_elevation), maximum_z
+                )
+                dz_bankfull = z_bankfull - z_water
+
+            # Add section info to the list for all sections
             widths["first_bank_i"].append(start_i)
             widths["last_bank_i"].append(stop_i)
             widths["widths"].append((stop_i - start_i) * resolution)
             widths["threshold"].append(dz_bankfull)
             widths["channel_count"].append(channel_count)
+        # Add threshold/width information as newcolumns to data frame
         for key in widths.keys():
             cross_sections[key] = widths[key]
-        # Record if the width is valid - only one possible channel that starts and ends
-        # within the samples
+        # Check valid - one channel, start & end within section, non-NaN thres
         valid_mask = cross_sections["channel_count"] == 1
         valid_mask &= cross_sections["first_bank_i"] > 0
         valid_mask &= cross_sections["last_bank_i"] < self.number_of_samples - 1
-        valid_mask &= cross_sections["threshold"] < maximum_threshold
         valid_mask &= numpy.logical_not(numpy.isnan(cross_sections["threshold"]))
+        # valid_mask &= cross_sections["threshold"] < maximum_threshold
         cross_sections["valid"] = valid_mask
 
     def fixed_threshold_width(
@@ -1269,7 +1505,7 @@ class ChannelCharacteristics:
 
                     # Detect banks - either ground above threshold, or no ground with
                     # vegetation over threshold
-                    if numpy.isnan(stop_i) and gnd_elevation_over_minimum > threshold:
+                    if gnd_elevation_over_minimum > threshold:
                         # Leaving the channel
                         stop_i = start_index + i
                 # work backward checking height
@@ -1277,21 +1513,19 @@ class ChannelCharacteristics:
                     gnd_elevation_over_minimum = veg_samples[start_index - i] - z_water
 
                     # Detect bank
-                    if numpy.isnan(start_i) and gnd_elevation_over_minimum > threshold:
+                    if gnd_elevation_over_minimum > threshold:
                         # Leaving the channel
                         start_i = start_index - i
                 # break if both edges detected
                 if not numpy.isnan(start_i) and not numpy.isnan(stop_i):
                     break
                 # break if both ends of the sampled cross section reached
-                if (
+                if (  # Set to max range
                     start_index + i >= self.number_of_samples - 1
                     and start_index - i <= 0
                 ):
-                    if numpy.isnan(start_i):
-                        start_i = 0
-                    if numpy.isnan(stop_i):
-                        stop_i = self.number_of_samples - 1
+                    start_i = 0
+                    stop_i = self.number_of_samples - 1
                     break
         return start_i, stop_i
 
